@@ -84,8 +84,12 @@ sub new {
 	return $self
 }
 
-sub DESTROY {
-	DEBUG( 100,"DESTROY context $_[0] callid=$_[0]->{callid}" ) if $_[0];
+# destroying of fields in perl5.8 cleanup can cause strange errors, where
+# it complains, that it cannot coerce array into hash. So use this function
+# on your own risks and rename it to DETSTROY if you want to have debugging
+# info
+sub _DESTROY {
+	DEBUG( 100,"DESTROY context $_[0] callid=$_[0]->{callid}" );
 }
 
 ############################################################################
@@ -111,6 +115,30 @@ sub peer {
 	return $data;
 }
 
+############################################################################
+# return list of outstanding requests matching filter, if no filter is given
+# returns all requests
+# Args: ($self,%filter)
+#  %filter
+#     method => name: filter for requests with given method
+#     request => packet: filter for packet, e.g. finds if packet is outstanding
+# Returns: @requests
+#   returns all matching requests (Net::SIP::Request objects), newest
+#   requests first
+############################################################################
+sub find_outstanding_requests {
+	my Net::SIP::Endpoint::Context $self = shift;
+	my %filter = @_;
+	my @trans = @{$self->{_transactions}} or return;
+	if ( my $pkt = $filter{request} ) {
+		@trans = grep { $pkt == $_->{request} } @trans or return;
+	}
+	if ( my $method = $filter{method} ) {
+		@trans = grep { $method eq $_->{request}->method } @trans or return;
+	}
+	return map { $_->{request} } @trans;
+}
+
 ############################################################################
 # creates a new SIP request packet within this context
 # Args: ($self,$method;$body,%args)
@@ -182,7 +210,6 @@ sub new_request {
 	return $request;
 }
 
-
 ############################################################################
 # set callback for context
 # Args: ($self,$cb)
@@ -243,10 +270,11 @@ sub handle_response {
 	# if response does not terminates transaction one need to add
 	# it again
 	my $tid = $response->tid;
+	my $method = $response->method;
 	my $trans = $self->{_transactions};
 	my (@ntrans,$tr);
 	foreach my $t (@$trans) {
-		if ( $t->{tid} eq $tid ) {
+		if ( !$tr and $t->{tid} eq $tid and $method eq $t->{request}->method) {
 			$tr = $t;
 		} else {
 			push @ntrans,$t
@@ -269,18 +297,6 @@ sub handle_response {
 		return;
 	};
 
-
-	# check if it's a response to the current method in the
-	# transaction (ACK,CANCEL,INVITE share the same transaction,
-	# but are different methods)
-
-	my $method = $tr->{request}->method;
-	$response->cseq =~m{^\d+\s+(\w+)};
-	if ($method ne $1 ) {
-		DEBUG( 10,"got response to method $1 but current method is $method. DROP" );
-		return;
-	}
-
 	my $cb = $tr->{callback};
 	my @arg = ($endpoint,$self);
 	my $code = $response->code;
@@ -334,21 +350,25 @@ sub handle_response {
 			# is response to INVITE, create ACK
 			# and propagate to upper layer
 			my $req = $tr->{request};
-			if ( my $contact = $response->get_header( 'contact' )) {
-				# 12.1.2 - set URI for dialog to contact given in response which
-				# establishes the dialog
-				$contact = $1 if $contact =~m{<(\w+:[^>\s]+)>};
-				$req->set_uri( $contact );
-			}
+
+			# create ACK, the ACK needs to be send with the original URI,
+			# even if the contact header (see below) changes the dialogs URI
 			my $ack = $req->create_ack( $response );
 			invoke_callback($cb,@arg,0,$code,$response,$leg,$from,$ack);
 			$endpoint->new_request( $ack,$self,undef,undef,leg => $leg, dst_addr => $from );
 
+			# 12.1.2 - set URI for dialog to contact given in response which
+			# establishes the dialog
+			if ( my $contact = $response->get_header( 'contact' )) {
+				$contact = $1 if $contact =~m{<(\w+:[^>\s]+)>};
+				$req->set_uri( $contact );
+			}
+
 			# use to-tag from this request to update 'to'
 			# FIXME: this should probably be better done by the upper layer
 			# which decides, which call to accept (in case of call-forking with
 			# multiple 2xx responses)
-			$self->{to} = $response->get_header( 'to' );
+			$self->{to} = $response->get_header( 'to' ) if ! $self->{incoming};
 
 		} else {
 			# response to ACK, REGISTER...
